/* prxtest - http proxy tester
 *
 * by scut / teso
 *
 */

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include "network.h"

#define	SCAN_RELAY	1
#define	SCAN_PROXY	2

void	usage (char *prgname);

void
usage (char *prgname)
{
	printf ("\n");
	printf ("prxtest - by scut / teso\n\n");
	printf ("usage: %s <-c|-p|-a> <ip>:<port>\n\n", prgname);
	printf ("\"-a\" will test the proxy at the `ip' on port `port' on both weaknesses\n");
	printf ("\"-c\" will test the proxy at the `ip' on port `port' for relay weaknesses,\n");
	printf ("\"-p\" will test the proxy at the `ip' on port `port' for proxy weaknesses\n");
	printf ("\"ip\" can be either a numerical quad-dotted IP or a hostname\n");
	printf ("\"port\" must be given, and must be a positive integer between 1 and 65535\n");
	printf ("\n");

	exit (1);
}

int
main (int argc, char **argv)
{
	char			*inb;
	bound			*tb;
	int			w, p;
	struct sockaddr_in	cs, cc;
	char			*ip;
	unsigned short		port;
	int			smode;

	if (argc != 3)
		usage (argv[0]);

	if (argv[1][0] == '-') {
		switch (argv[1][1]) {
		case 'a':
		case 'c':
			smode = SCAN_RELAY;
			break;
		case 'p':
			smode = SCAN_PROXY;
			break;
		default:
			usage (argv[0]);
		}
	} else
		usage (argv[0]);

	if (net_parseip (argv[2], &ip, &port) == 0)
		usage (argv[0]);

	/* weirdy, mem-lacking code, but since the program is started
	 * on a per-host base, this doesn't really matter, the second
	 * port will be deallocated automatically on program exit
	 */

nhost:
	tb = net_bind ("*", 0);
	if (tb == NULL) {
		fprintf (stderr, "binding failed\n");
		exit (1);
	}

	printf ("%s:%hu", ip, port);
	fflush (stdout);

	printf (" -C");
	fflush (stdout);

	w = net_connect (&cc, ip, port, NULL, 0, 30);
	if (w == -1) {
		printf (" FAILED TO CONNECT\n");
		exit (0);
	} else {
		printf ("-L");
		fflush (stdout);
	}

	if (smode == SCAN_RELAY) {
		net_write (w, "CONNECT %s:%hu HTTP/1.0\n\n", net_getlocalip (), tb->port);
		printf ("-C");
	} else if (smode == SCAN_PROXY) {
		net_write (w, "GET http://%s:%hu/index.html HTTP/1.0\n\n", net_getlocalip (), tb->port);
		printf ("-G");
	}
	fflush (stdout);

	p = net_accept (tb->bs, &cs, 20);
	if (p <= 0) {
		printf ("- FAILED\n");
		goto nend2;
	}
	printf ("-O");
	fflush (stdout);

	if (smode == SCAN_RELAY) {
		net_write (w, "GET\n");
	}

	if (net_rlineta (p, &inb, 25) <= 0) {
		printf ("- OPEN, BUT BROKEN\n");
		goto nend1;
	}
	if (strncmp (inb, "GET", 3) == 0) {
		printf ("- OPEN, %s\n", (smode == SCAN_RELAY) ? "RELAYS" : "PROXY");
	} else {
		printf ("- OPEN, GARBAGE (?)\n");
	}

nend1:
	close (p);
nend2:
	close (w);
	free (inb);

	if (argv[1][1] == 'a' && smode == SCAN_RELAY) {
		smode = SCAN_PROXY;
		goto nhost;
	}

	return (0);
}

